@@ -13,7 +13,6 @@ if TYPE_CHECKING:  # pragma: no cover
 
     class DataclassType:
         __pydantic_model__: Type[BaseModel]
-        __post_init_original__: Callable[..., None]
         __initialised__: bool
 
         def __init__(self, *args: Any, **kwargs: Any) -> None:
@@ -24,14 +23,6 @@ if TYPE_CHECKING:  # pragma: no cover
             pass
 
 
-def _pydantic_post_init(self: 'DataclassType') -> None:
-    d = validate_model(self.__pydantic_model__, self.__dict__, cls=self.__class__)
-    object.__setattr__(self, '__dict__', d)
-    object.__setattr__(self, '__initialised__', True)
-    if self.__post_init_original__:
-        self.__post_init_original__()
-
-
 def _validate_dataclass(cls: Type['DataclassType'], v: Any) -> 'DataclassType':
     if isinstance(v, cls):
         return v
@@ -69,16 +60,26 @@ def _process_class(
     config: Type['BaseConfig'],
 ) -> 'DataclassType':
     post_init_original = getattr(_cls, '__post_init__', None)
+    post_init_post_parse = getattr(_cls, '__post_init_post_parse__', None)
     if post_init_original and post_init_original.__name__ == '_pydantic_post_init':
         post_init_original = None
+
+    def _pydantic_post_init(self: 'DataclassType', *initvars: Any) -> None:
+        if post_init_original is not None:
+            post_init_original(self, *initvars)
+        d = validate_model(self.__pydantic_model__, self.__dict__, cls=self.__class__)[0]
+        object.__setattr__(self, '__dict__', d)
+        object.__setattr__(self, '__initialised__', True)
+        if post_init_post_parse is not None:
+            post_init_post_parse(self)
+
     _cls.__post_init__ = _pydantic_post_init
     cls = dataclasses._process_class(_cls, init, repr, eq, order, unsafe_hash, frozen)  # type: ignore
 
     fields: Dict[str, Any] = {
-        name: (field.type, field.default if field.default != dataclasses.MISSING else Required)
-        for name, field in cls.__dataclass_fields__.items()
+        field.name: (field.type, field.default if field.default != dataclasses.MISSING else Required)
+        for field in dataclasses.fields(cls)
     }
-    cls.__post_init_original__ = post_init_original
 
     validators = gather_validators(cls)
     cls.__pydantic_model__ = create_model(
